//+------------------------------------------------------------------+
//|                                                 Woodie's CCI.mq4 |
//|                                                  Vadim Shumiloff |
//|                                                shumiloff@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Vadim Shumiloff"
#property link      "shumiloff@mail.ru"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Gray        //     
#property indicator_color2 Gold        //  "" ,    
#property indicator_color3 Green       //    
#property indicator_color4 Red         //    
#property indicator_color5 DeepSkyBlue //  CCI
#property indicator_color6 Yellow      //  Turbo CCI

//  
extern int CCIPeriod = 14;
extern int TurboCCIPeriod = 6;
extern int GoldPeriod = 6;

double CCI[];
double CCIRed[];
double CCIGreen[];
double CCIGray[];
double CCIGold[];
double TurboCCI[];

//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
int init()
{
   string short_name;
   if(CCIPeriod <= 0) CCIPeriod = 14;
   if(TurboCCIPeriod <= 0) TurboCCIPeriod = 6;
   IndicatorBuffers(6);

   SetIndexStyle(0, DRAW_HISTOGRAM);
   SetIndexBuffer(0, CCIGray);
   SetIndexDrawBegin(0, CCIPeriod*2);
   
   SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(1, CCIGold);
   SetIndexDrawBegin(1, CCIPeriod*2);

   SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(2, CCIGreen);
   SetIndexDrawBegin(2, CCIPeriod*2);
   
   SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(3, CCIRed);
   SetIndexDrawBegin(3, CCIPeriod*2);
   
   SetIndexStyle(4, DRAW_LINE, STYLE_SOLID, 3);
   SetIndexBuffer(4, CCI);
   SetIndexDrawBegin(4, CCIPeriod*2);
      
   SetIndexStyle(5, DRAW_LINE);
   SetIndexBuffer(5, TurboCCI);
   SetIndexDrawBegin(5, CCIPeriod*2);
         
   short_name="Woodie's CCI(" + CCIPeriod + ", " + TurboCCIPeriod + ")";
   IndicatorShortName(short_name);
   //IndicatorShortName("Woodie's CCI");
   SetIndexLabel(0, short_name);

   return(0);
}

int deinit()
{
   Comment("");
}

//+------------------------------------------------------------------+
//| Woodie's Commodity Channel Index                                 |
//+------------------------------------------------------------------+
int start()
{
   int    i, counted_bars = IndicatorCounted();
   double price, sum, mul; 
   if(CCIPeriod <= 1) return(0);
   if(Bars <= CCIPeriod) return(0);

   int limit = Bars - counted_bars;
   if(counted_bars > 0)
   { 
      limit++;
      for(i=0; i<limit; i++) Calculate(i);
   }
   else
   {
      // first calculation
      for(i = 1; i <= CCIPeriod; i++) CCI[Bars-i] = 0.0;
      for(i=Bars-CCIPeriod; i>=0; i--) Calculate(i);
   }
   Comment("Ready: ", ReadyBar(), "%");
   return(0);
}
//+---------------------------------------------------------------------------------------------------+

int ReadyBar()
{
   int TF = Time[0] - Time[1];
   int DT = TimeCurrent() - Time[0];
   return (MathFloor(DT*100/TF));
}

void Calculate(int n)
{
   TurboCCI[n] = iCCI(Symbol(), 0, TurboCCIPeriod, PRICE_TYPICAL, n); 
   CCI[n] = iCCI(Symbol(), 0, CCIPeriod, PRICE_TYPICAL, n); 

   int LastAnotherSign = 7;
   bool plus = (CCI[n] > 0);
   bool sign;
   for (int i=1; i <= GoldPeriod; i++)
   {
      sign = (CCI[n+i] > 0);
      if (sign != plus)
      {
         LastAnotherSign = i;
         break;
      }
   }

   if (LastAnotherSign > GoldPeriod && plus)
   {
      CCIGreen[n] = CCI[n];
      CCIRed[n] = 0.0;
      CCIGray[n] = 0.0;
      CCIGold[n] = 0.0;
   }
   else
   {
      if (LastAnotherSign > GoldPeriod && (!plus))
      {
         CCIRed[n] = CCI[n];
         CCIGreen[n] = 0.0;
         CCIGray[n] = 0.0;
         CCIGold[n] = 0.0;
      }
      else
      {
         if (LastAnotherSign == GoldPeriod)
         {
            CCIRed[n] = 0.0;
            CCIGreen[n] = 0.0;
            CCIGray[n] = 0.0;
            CCIGold[n] = CCI[n];
         }
         else
         {
            CCIRed[n] = 0.0;
            CCIGreen[n] = 0.0;
            CCIGray[n] = CCI[n];
            CCIGold[n] = 0.0;
         }
      } 
   } 
}